#include <BasicAbstractions/Font.h>
#include <States/GameState/GameState.h>
#include <States/SelectState/SelectState.h>

void GameState::event_handling() {
    sf::Event event{};
    while (m_window.pollEvent(event)) {
        if (event.type == sf::Event::Closed) {
            m_state_manager.set_next_state(std::make_unique<SelectState>(m_state_manager, config::SELECT_LEVEL_VIDEO_MODE, config::SELECT_LEVEL_TITLE));
        }
        if (event.type == sf::Event::KeyPressed && event.key.code == sf::Keyboard::Z && event.key.control)
            m_context_manager.restore_previous_context();
        if (event.type == sf::Event::KeyPressed && m_context_manager.get_current_context().state == GameContext::INGAME)
            process_key_pressed(event.key.code);
    }
}

void GameState::update() {
    if (m_context_manager.get_current_context().state != GameContext::INGAME)
        return;
    std::vector<std::unique_ptr<IGameEvent>> events;

    for (const auto& obj : m_context_manager.get_current_context().dynamic_objects) {
        obj->action();
        if (auto event_ptr = obj->accept(&m_context_manager.get_current_context().pacman); event_ptr != nullptr)
            events.emplace_back(std::move(event_ptr));
    }

    for (const auto& obj : m_context_manager.get_current_context().static_objects) {
        if (auto event_ptr = obj->accept(&m_context_manager.get_current_context().pacman); event_ptr != nullptr)
            events.emplace_back(std::move(event_ptr));
    }

    if (m_context_manager.get_current_context().static_objects.empty()) {
        events.emplace_back(std::make_unique<WinGame>());
    }

    m_context_manager.get_current_context().pacman.move();

    for (const auto& event : events)
        event->handle(m_context_manager.get_current_context());
}

void GameState::render() {
    sf::Text text;
    sf::RectangleShape rect;
    switch (m_context_manager.get_current_context().state) {
        case GameContext::INGAME:
            m_window.clear(config::GAME_COLOR_BACKGROUND_INGAME);
            break;
        case GameContext::LOST:
            text.setFont(MyFont::Instance());
            text.setFillColor({255, 0, 0, 220});
            text.setString("LOST!");
            text.setCharacterSize(300);
            text.setStyle(sf::Text::Bold);
            text.setPosition(
                round((m_window.getSize().x - text.getLocalBounds().width) / 2 - text.getLocalBounds().left),
                round((m_window.getSize().y - text.getLocalBounds().height) / 2 - text.getLocalBounds().top)
            );
            rect.setSize(sf::Vector2f(text.getLocalBounds().width + 100, text.getLocalBounds().height + 100));
            rect.setFillColor(sf::Color(240, 0, 0, 50));
            rect.setOutlineThickness(5);
            rect.setOutlineColor(sf::Color::White);
            rect.setPosition(
                round((m_window.getSize().x - rect.getSize().x) / 2),
                round((m_window.getSize().y - rect.getSize().y) / 2)
            );
            m_window.clear(config::GAME_COLOR_BACKGROUND_INGAME);
            break;
        case GameContext::WIN:
            m_window.clear(config::GAME_COLOR_BACKGROUND_WIN);
            break;
    }
    m_maze.draw_into(m_window);
    for (const auto& obj : m_context_manager.get_current_context().static_objects)
        obj->draw_into(m_window);
    m_context_manager.get_current_context().pacman.draw_into(m_window);
    for (const auto& obj : m_context_manager.get_current_context().dynamic_objects)
        obj->draw_into(m_window);
    m_window.draw(rect);
    m_window.draw(text);
    m_window.display();
}

void GameState::process_key_pressed(const sf::Keyboard::Key key) {
    switch (key) {
        case config::KEY_UP:
            m_context_manager.save_context();
            m_context_manager.get_current_context().pacman.replace(Room::UP);
            break;
        case config::KEY_DOWN:
            m_context_manager.save_context();
            m_context_manager.get_current_context().pacman.replace(Room::DOWN);
            break;
        case config::KEY_LEFT:
            m_context_manager.save_context();
            m_context_manager.get_current_context().pacman.replace(Room::LEFT);
            break;
        case config::KEY_RIGHT:
            m_context_manager.save_context();
            m_context_manager.get_current_context().pacman.replace(Room::RIGHT);
            break;
        default: break;
    }
}

bool GameState::do_step() {
    event_handling();
    update();
    render();
    return true;
}
